@@ -7,7 +7,7 @@ from fastapi.requests import Request
 from fastapi.responses import HTMLResponse
 from requests import Response as ResponseType
 
-from fastel.collections import get_site_config
+from fastel.collections import get_collection, get_site_config
 from fastel.config import SdkConfig
 from fastel.exceptions import APIException
 from fastel.logistics.common.models import (
@@ -36,6 +36,7 @@ class EcpayLogisticsCreate:
             merchant_id=SdkConfig.ecpay_logistics_merchant_id,
             hash_key=SdkConfig.ecpay_logistics_hash_key,
             hash_iv=SdkConfig.ecpay_logistics_hash_iv,
+            stage=SdkConfig.stage,
         )
         self.server_reply_url = callback_url
 
@@ -65,8 +66,18 @@ class EcpayLogisticsCreate:
             APIException(
                 status_code=400, error="validation_error", detail="物流訂單商品不能超過20000元新台幣"
             )
+        counter = get_collection("counter").find_one_and_update(
+            {"collection": "logistics", "order_id": order.order_id},
+            {
+                "$inc": {"idx": 1},
+                "$setOnInsert": {"collection": "logistics", "order_id": order.order_id},
+            },
+            upsert=True,
+        )
+        if not counter:
+            counter = {"idx": 0}
         common_props = dict(
-            MerchantTradeNo=f"{order.order_id}_{len(order.logistics) + 1}",
+            MerchantTradeNo=f"{order.order_id}_{counter['idx'] + 1}",
             GoodsName=f"訂單{order.order_id}",
             GoodsAmount=order.total,
             ReceiverName=self._replace_limit_name(name=order.receiver_name, limit=10),
@@ -152,9 +163,9 @@ class EcpayLogisticsCreate:
 class EcpayLogisticsCallback:
     def __init__(self) -> None:
         self.gateway = EcpayLogistics(
-            SdkConfig.ecpay_merchant_id,
-            SdkConfig.ecpay_hash_key,
-            SdkConfig.ecpay_hash_iv,
+            merchant_id=SdkConfig.ecpay_logistics_merchant_id,
+            hash_key=SdkConfig.ecpay_logistics_hash_key,
+            hash_iv=SdkConfig.ecpay_logistics_hash_iv,
             stage=SdkConfig.stage,
         )
         self.response = "0|NOPE"
@@ -282,23 +293,27 @@ class EcpayLogisticsCallback:
 class EcpayGetShippingNote:
     def __init__(self) -> None:
         self.gateway = EcpayLogistics(
-            SdkConfig.ecpay_merchant_id,
-            SdkConfig.ecpay_hash_key,
-            SdkConfig.ecpay_hash_iv,
+            merchant_id=SdkConfig.ecpay_logistics_merchant_id,
+            hash_key=SdkConfig.ecpay_logistics_hash_key,
+            hash_iv=SdkConfig.ecpay_logistics_hash_iv,
             stage=SdkConfig.stage,
         )
 
     def __call__(self, data: Dict[str, Any]) -> Tuple[bytes, str]:
         validated = self.validate(data)
         parsed = self.parse_payload(validated)
-        html = self.gateway.get_shipping_note(parsed)
-        img_url = self.fetch_html_img(html)
-        print("shipping note url:", img_url)
-        img_url = img_url.replace("amp;", "")
-        img_resp: ResponseType = requests.get(img_url)
-        img_resp.raise_for_status()
-        img = img_resp.content
-        content_type = img_resp.headers.get("Content-Type", "image/png")
+        html, logistics_subtype = self.gateway.get_shipping_note(parsed)
+        print(html)
+        if logistics_subtype == "UniMartC2C":
+            return html.encode(), "text/html"
+        else:
+            img_url = self.fetch_html_img(html)
+            print("shipping note url:", img_url)
+            img_url = img_url.replace("amp;", "")
+            img_resp: ResponseType = requests.get(img_url)
+            img_resp.raise_for_status()
+            img = img_resp.content
+            content_type = img_resp.headers.get("Content-Type", "image/png")
         return img, content_type
 
     @staticmethod
@@ -541,6 +556,7 @@ class EcpayLogisticsMap:
             merchant_id=SdkConfig.ecpay_logistics_merchant_id,
             hash_key=SdkConfig.ecpay_logistics_hash_key,
             hash_iv=SdkConfig.ecpay_logistics_hash_iv,
+            stage=SdkConfig.stage,
         )
         self.server_reply_url = callback_url
 
